<?php

namespace Teknisa\SUP\Controllers;

use Teknisa\SUP\Controllers\AFProcessController as AFProcess;
use Teknisa\SUP\Controllers\ApprovalWorkflowController as WorkflowUtil;
use Teknisa\SUP\Controllers\OrderAFController as OrderAF;
use Teknisa\SUP\Controllers\ProductsAFController as ProductAF;
use Teknisa\SUP\Controllers\ReportOrderController as ReportOrder;
use Teknisa\SUP\Services\ConnectionIntegraWebService;
use Teknisa\SUP\Util\XMLUtil;

class AutomaticImportAF
{   

    private $ordersErrorsCode = [];

    public function __construct(OrderAF $orderAF, ProductAF $productAF, AFProcess $afProcess, WorkflowUtil $workFlowUtil, ReportOrder $reportOrder, ConnectionIntegraWebService $webService, XMLUtil $xmlUtil)
    {
        $this->orderAF = $orderAF;
        $this->productAF = $productAF;
        $this->afProcess = $afProcess;
        $this->xmlUtil = $xmlUtil;
        $this->reportOrder = $reportOrder;
        $this->connectWebService = $webService;
        $this->workFlowUtil = $workFlowUtil;
    }

    public function initializeProccess()
    {
        echo nl2br("\nIniciando processo de integração\n");
        $services = $this->connectWebService->getAvailableServices();
        foreach ($services as $service) {
            $supplierName = $service['NMFANTFORN'];
            $supplierHost = $service['DSHOST'];
            echo nl2br("\nEstabelecendo conexão com $supplierName - $supplierHost");
            $response = $this->connectWebService->connectWebService($service);
            if ($response) {
                $this->xmlUtil->parseStringXMLToObject($response);
                self::importAFAuto($response);
            } else {
                echo nl2br("\nErro de conexão com $supplierName - $supplierHost");
            }
        }
        echo nl2br("\n\nFim do processo!"); 
    }

    public function importAFAuto($xmlObject)
    {
        if (self::checkXMLObjectIsValid($xmlObject)) {
            $cdSupplier = "J".(string)$xmlObject->info->{'cnpj-remetente'}[0];
            self::startProccess($xmlObject, $cdSupplier);
        }
    }

    private function checkXMLObjectIsValid($xmlObject)
    {
        if(isset($xmlObject->mensagem)) {
            echo nl2br("\n".(string)$xmlObject->mensagem);
            return false;
        } else if(!isset($xmlObject->dados)) {
            echo nl2br("\n"."Erro de conexão.");
            return false;
        } else {
            return true;
        }
    }

    private function startProccess($xmlObject, $cdSupplier)
    {
        $orders = $xmlObject->dados->pedido;
        foreach ($orders as $order) {
            echo nl2br("\nIniciando importação do pedido: ".(string)$order->numero);
            $dataSetErrors = self::validatesXMLData($order, $cdSupplier);
            self::importOrder($order, $dataSetErrors, $cdSupplier);
        }
        self::sendReportEmail($cdSupplier);
    }

    public function validatesXMLData($order, $cdSupplier)
    {
        $dataSetErrors = [];
        $dataSetErrors['order'] = $this->orderAF->validatesDataAF($order, $cdSupplier);
        $dataSetErrors['products'] = $this->productAF->validatesDataProduct($order, $cdSupplier);

        return $dataSetErrors;
    }

    public function importOrder($order, $dataSetErrors, $cdSupplier)
    {
        $filteredProducts = self::filterProducts($dataSetErrors['products'], $order);
        if (empty($dataSetErrors['order']) && !empty($filteredProducts)) {
            self::importAF($order, $cdSupplier, $filteredProducts);
            if(!empty($dataSetErrors['products'])) {
                self::saveImportErrors($order, $dataSetErrors, $cdSupplier);
            }
        } else {
            self::saveImportErrors($order, $dataSetErrors, $cdSupplier);
        }
    }

    public function filterProducts($productErrors, $order)
    {
        $products = $this->productAF->getProductArray($order);
        for($cont=0; $cont < count($products); $cont++) {
            $cdProduct = (string)$products[$cont]->codcliente[0];
            foreach($productErrors as $cdProductError=>$error) {
                if($cdProductError == $cdProduct) {
                    unset($products[$cont]);
                    $cont--;
                }
            }
        }
        return $products;
    }

    private function importAF($order, $cdSupplier, $filteredProducts) {
        $dataToSave = self::getDataToSave($order);
        self::saveAF($order, $dataToSave, $cdSupplier, $filteredProducts);
        self::insertOrderOnWorkflow($dataToSave['cdFildest'], $dataToSave['nrAutoForn']);
    }

    public function getDataToSave($order)
    {
        return [
            'nrAutoForn' => $this->orderAF->getNewCodeAutoForne(),
            'orderDate' => (string) $order->{'data-pedido'},
            'cdFildest' => (string) $order->{'cod-local'},
        ];
    }

    public function saveAF($order, $dataToSave, $cdSupplier, $filteredProducts)
    {
        $this->orderAF->saveOrder($dataToSave, $cdSupplier);
        $this->productAF->saveProduct($filteredProducts, $dataToSave);
        $this->afProcess->processesAF($order, $dataToSave, $cdSupplier);
    }

    public function insertOrderOnWorkflow($cdFilial, $nrAutoForn)
    {
        $param = $this->workFlowUtil->utilizaWorkflow($cdFilial);
        if ($param == 'S') {
            $cdFildest = '9999';
            $operation = 'LANCAMENTO';
            $orderType = 'AF';
            $insertWorkflow = $this->workFlowUtil->insertOrderOnWorkflow($cdFilial, $cdFildest, $nrAutoForn, $operation, $orderType);
            echo nl2br("\nPedido inserido no Workflow de Aprovação");
        } else {
            echo nl2br("\nUnidade $cdFilial não utiliza Workflow de Aprovação");
        }
    }

    private function saveImportErrors($order, $dataSetErrors, $cdSupplier)
    {
        $numOrder = (string) $order->numero;
        $dataToSave = self::getDataToSave($order);
        if(!empty($dataSetErrors)) {
            $this->afProcess->saveImportErrors($order, $dataSetErrors);
            $this->reportOrder->reportOrderError($numOrder, $cdSupplier);
            array_push($this->ordersErrorsCode, ['numOrder' => $numOrder, 'cdFildest' => $dataToSave['cdFildest']]);
        }
        echo nl2br("\nErro no pedido ".$numOrder);
    }

    private function sendReportEmail($cdSupplier) {
        if(!empty($this->ordersErrorsCode)) {
            $this->reportOrder->sendReportMail($cdSupplier, $this->ordersErrorsCode);
        }
    }

}
